import 'babel-polyfill';
import React from 'react';
import PropTypes from 'prop-types';
import { Provider } from 'react-redux';
import { IntlProvider } from 'react-intl';
import { reducer as form } from 'redux-form';
import classnames from 'classnames';
import _ from 'lodash';

import { storiesOf } from '@storybook/react';
import { withInfo } from '@storybook/addon-info';

import configureStore from '../configureStore';
import {
    Conversation,
    Conversations,
    MessageForm,
    Messages,
    reducer as chatReducer,
    constants as chatConstants,
    actionTypes as chatActionTypes,
} from '../../src/modules/chat';

import { firebaseData, intlMessages } from './chatData';

import './chat.story.scss';
import '../../src/modules/chat/styles.scss';

const state = {
    chat: {
        page: 1,
        amount: 5,
        sentMessages: 0,
        firstUnreadMessageId: '-L7U1C3aNgrVDHKD_75h',
    },
    firebase: firebaseData,
};
const reducers = {
    form,
    chat: chatReducer,
    firebase: (s = firebaseData) => s,
};

const logMessageFormActions = () => {
    return next => action => {
        if (action.type === 'FORM_SUBMIT' && action.form === chatConstants.NEW_MESSAGE_FORM_ID) {
            const additionalInfo = `for conversation ${action.data.conversationId} and user ${action.data.userId}`;
            // eslint-disable-next-line no-console
            console.info('SENDING CHAT MESSAGE ', `"${action.data.text}"`, additionalInfo);
        }

        if (action.type === chatActionTypes.CREATE_IMAGE_MESSAGE) {
            const additionalInfo = `for conversation ${action.data.conversationId} and user ${action.data.userId}`;
            // eslint-disable-next-line no-console
            console.info('SENDING CHAT IMAGE', action.image, additionalInfo);
        }

        next(action);
    };
};

const store = configureStore(state, reducers, logMessageFormActions);

const SearchIcon = () => (
    <svg width="18px" height="18px" viewBox="0 0 18 18" version="1.1" xmlns="http://www.w3.org/2000/svg">
        <g id="Symbols" stroke="none" strokeWidth="1" fill="none" fillRule="evenodd">
            <g id="ic_search_24px" transform="translate(-3.000000, -3.000000)">
                <path
                    d="M15.5,14 L14.71,14 L14.43,13.73 C15.41,12.59 16,11.11 16,9.5 C16,5.91 13.09,3 9.5,3 C5.91,3 3,5.91 3,9.5 C3,13.09 5.91,16 9.5,16 C11.11,16 12.59,15.41 13.73,14.43 L14,14.71 L14,15.5 L19,20.49 L20.49,19 L15.5,14 Z M9.5,14 C7.01,14 5,11.99 5,9.5 C5,7.01 7.01,5 9.5,5 C11.99,5 14,7.01 14,9.5 C14,11.99 11.99,14 9.5,14 Z"
                    id="Shape"
                    fill="#9BC111"
                    fillRule="nonzero"
                />
                <polygon id="Shape" points="0 0 24 0 24 24 0 24" />
            </g>
        </g>
    </svg>
);

const UploadIcon = () => (
    <svg version="1.1" xmlns="http://www.w3.org/2000/svg" x="0px" y="0px" viewBox="0 0 53.3 53.4">
        <g>
            <path d="M44.1,32.5c-0.6,0-1.2,0.5-1.2,1.2v8c0,0.1-0.1,0.1-0.2,0.1H10.6c-0.1,0-0.2-0.1-0.2-0.1v-8c0-0.6-0.5-1.2-1.2-1.2
                C8.6,32.5,8,33,8,33.6v8c0,1.4,1.1,2.5,2.5,2.5h32.2c1.4,0,2.5-1.1,2.5-2.5v-8C45.3,33,44.7,32.5,44.1,32.5z"
            />
            <path d="M17.4,21.2l8-8v21c0,0.6,0.5,1.2,1.2,1.2c0.6,0,1.2-0.5,1.2-1.2v-21l8,8c0.2,0.2,0.5,0.3,0.8,0.3c0.3,0,0.6-0.1,0.8-0.3
                c0.5-0.5,0.5-1.2,0-1.7l-10-10c-0.2-0.2-0.5-0.3-0.8-0.3c-0.3,0-0.6,0.1-0.8,0.3l-10,10c-0.5,0.5-0.5,1.2,0,1.7
                C16.2,21.7,17,21.7,17.4,21.2z"
            />
        </g>
    </svg>
);

const ConversationsItem = ({ userId, conversationId, users }) => (
    <div className="content">
        {conversationId} {_.get(_.find(users, (__, id) => id !== userId), 'name')}
    </div>
);

ConversationsItem.propTypes = {
    userId: PropTypes.string.isRequired,
    conversationId: PropTypes.string.isRequired,
    users: PropTypes.objectOf(
        PropTypes.shape({
            name: PropTypes.string,
        })
    ).isRequired,
};

const Author = ({ author, seen }) => (
    <div className={classnames('author-circle', { 'author--unseen': !seen })}>
        {_.get(author, 'name', '').split(' ').map(s => s.charAt(0)).join('')}
    </div>
);

Author.propTypes = {
    author: PropTypes.shape({
        name: PropTypes.string.isRequired,
    }).isRequired,
    seen: PropTypes.bool.isRequired,
};

storiesOf('chat/Conversations', module)
    .addDecorator(story => (
        <Provider store={store}>
            <IntlProvider locale="en" messages={intlMessages}>
                <div className="panel conversations-list-panel">{story()}</div>
            </IntlProvider>
        </Provider>
    ))
    .add('basic', withInfo()(() => {
        return (
            <Conversations ConversationsItem={ConversationsItem} userId="dispatcher_9" />
        );
    }))
    .add('with search bar', withInfo()(() => {
        return (
            <Conversations
                ConversationsItem={ConversationsItem}
                userId="dispatcher_9"
                searchEnabled
                searchLabel={<SearchIcon />}
            />
        );
    }));

storiesOf('chat/MessageForm', module)
    .addDecorator(story => (
        <Provider store={store}>
            <IntlProvider locale="en" messages={intlMessages}>
                <div className="panel messages-panel">
                    {story()}
                </div>
            </IntlProvider>
        </Provider>
    ))
    .add('basic', withInfo()(() => {
        return (
            <MessageForm
                userId="dispatcher_9"
                conversationId="driver_1551"
                sendLabel="Send it!"
            />
        );
    }))
    .add('custom input', withInfo()(() => {
        return (
            <MessageForm
                userId="dispatcher_9"
                conversationId="driver_1551"
                sendLabel="Send it!"
                newMessageInput="textarea"
            />
        );
    }))
    .add('with sending files', withInfo()(() => {
        return (
            <MessageForm
                userId="dispatcher_9"
                conversationId="driver_1551"
                sendLabel={<strong>Send it!</strong>}
                uploadLabel={<span><UploadIcon />UPLOAD FILE..</span>}
                filesEnabled
            />
        );
    }));

storiesOf('chat/Messages', module)
    .addDecorator(story => (
        <Provider store={store}>
            <IntlProvider locale="en" messages={intlMessages}>
                <div className="panel messages-panel">
                    {story()}
                </div>
            </IntlProvider>
        </Provider>
    ))

    .add('basic', withInfo()(() => {
        return (
            <Messages conversationId="driver_1551" userId="dispatcher_9" />
        );
    }))
    .add('custom author', withInfo()(() => {
        return (
            <div className="custom-author-messages">
                <Messages
                    conversationId="driver_1551"
                    userId="dispatcher_9"
                    MessageAuthor={Author}
                />
            </div>
        );
    }));


storiesOf('chat/Conversation', module)
    .addDecorator(story => (
        <Provider store={store}>
            <IntlProvider locale="en" messages={intlMessages}>
                <div className="panel messages-panel">
                    {story()}
                </div>
            </IntlProvider>
        </Provider>
    ))
    .add('basic', withInfo()(() => {
        return (
            <Conversation
                conversationId="driver_1551"
                userId="dispatcher_9"
                sendLabel="Send it!"
            />
        );
    }));
